#ifndef TWIL_CONTROLLERS_CART_LINEARIZING_CONTROLLER_H
#define TWIL_CONTROLLERS_CART_LINEARIZING_CONTROLLER_H

#include <cstddef>
#include <vector>
#include <string>

#include <ros/node_handle.h>
#include <hardware_interface/joint_command_interface.h>
#include <controller_interface/controller.h>
#include <std_msgs/Float64MultiArray.h>
#include <sensor_msgs/JointState.h>

#include <Eigen/Dense>

namespace twil_controllers
{
	class CartLinearizingController: public controller_interface::Controller<hardware_interface::EffortJointInterface>
	{
		public:
			CartLinearizingController(void);
			~CartLinearizingController(void);
		
			bool init(hardware_interface::EffortJointInterface *robot, ros::NodeHandle &n);
			void starting(const ros::Time& time);
			void update(const ros::Time& time,const ros::Duration& duration);
				
		private:
			ros::NodeHandle node_;
			hardware_interface::EffortJointInterface *robot_;
			std::vector<hardware_interface::JointHandle> joints_;
		
			void commandCB(const std_msgs::Float64MultiArray::ConstPtr &command);
			ros::Subscriber sub_command_;
			
			Eigen::Vector2d nu;
			Eigen::Vector2d torque;
			Eigen::Vector2d v;
			Eigen::Vector2d u;
			
			Eigen::Matrix2d Ginv;
			Eigen::Matrix2d F;
		
			std::vector<double> wheelRadius;
			double wheelBase;
	};
}
#endif
